function lpformats
% Intruduction to MAT, MPS and LPP formats supported by LIPSOL
% ============================================================
%
%LIPSOL can read LP problems in 3 formats: MAT, MPS and LPP.  A
%MAT-file should have an extension .mat and in general should be
%stored in the directory matdir; similarly for MPS and LPP-files. 
%One can store problems in other directories as long as they are
%specified in the configuration function solvers/lib/lipconfig.m 
%as entries of the "string vector" PROBDIR (see the file for more
%details).  Problem files can be either uncompressed or compressed.
%
%If there exist, say, foo.mat, foo.mps and/or foo.lpp, LIPSOL will
%choose to solve one according to the order of MAT, MPS and LPP.
%
%MAT-files:
%-------------
%A MAT-file should consist of the following 7 quantities:
%           A b c lbounds ubounds BIG NAME
%representing the linear program
%        min           c'*x
%        s.t.         Ax = b
%             lbounds <= x <= ubounds
%NAME is a string storing the name of a linear program, and BIG is a
%large number; ubounds(i) = BIG if no upper-bound for x(i) exists.
%
%MPS-files:
%-------------
%MPS-format is a de facto industrial standard for describing linear
%programs, especially the large ones.  For more details, see the
%Linear Programming FAQ, maintained by John Gregory (jwg@cray.com),
%which can be found at the Web site: http://www.skypoint.com/
%subscribers/ashbury/linear-programming-faq.html
%
%LPP-files:
%-------------
%LPP stands for LP-Plain, a What-You-See-Is-What-You-Get format.
%It is particularly useful for small problems and classroom use. 
%Here is an LPP-file called, say, example.lpp:
%
%% This is a simple example.
%NAME: example
%objective:
%  min 2x + 5y - 2.5z
%constraints:
%  1) x + {sin(pi/4)/4} z < 5
%  2) y + z > {exp(2)} x
%bounds:
%  1) -2 < x < 2
%  2) y > 1
%  3) z < 3
%
%An LPP-file consists of 4 groups: name, objective, constraints
%and bounds.  These markers must be present even when one group
%is empty (e.g., no bounds).  Certain restrictions apply:
%
%   1) A colon ":" must appear before the problem name. 
%   2) Either "min" or "max" should precede the objective.
%   3) A number and a ")" must precede a constraint/bound.
%   4) "<" and ">" are used instead of "<=" and ">=".
%   5) A coefficient must be either a number or a Matlab
%      expressions enclosed in a pair of braces.
%   6) "*" and "/" are not allowd except inside braces.
%   7) There must be one and only one variable per bound,
%      without coefficient or sign whatsoever (e.g., 2x < 1 
%      and x + y > 1 are illegal as bounds; 2 > z is alright
%      but -2 < -z is not; 1 < x < 1 is ok but x = 1 is not).
%
%Zero-lower bounds are implicitly assumed unless otherwise specified.   
%LPP format is case-insensitive and imposes essentially no restriction 
%on spacing as long as it makes sense mathematically.
%
%(Command LPFORMATS gives a paged view of this file.)

% Yin Zhang, July 1995, Dept Math/Stat, UMBC

more on; help lpformats; more off;
