c
c Read input file in MPS format and write output file in MAT format:
c
c     A  b  c  l  u
c
c     for the linear programming problem
c
c        min  c'x
c        Ax=b
c        l .le. x .le. u
c
c where A is in sparse form.
c
      program          mps2mat
      integer          mmax,nmax,namax
      parameter        (mmax=30000,nmax=60000,namax=900000)
C  =============================================
      integer          freemax
      parameter        (freemax=5000)
C  =============================================
      logical          rdc2,lsort,objdone
      integer          m,n,na,n0,n1,krow(mmax),ia(namax),ja(nmax),
     *                 irow(mmax),jcol(nmax),i,j,i0,ij
      character*1      test(3)
      character*8      c1,r1,r2,c0,
     *                 rownam(mmax),colnam(nmax),objnam,junk
      character*80     title
      double precision A(namax),b(mmax),c(nmax),l(nmax),u(nmax),
     *                 v1,v2,mone,zero,one,big
C  =============================================
      INTEGER*4        matOpen, matClose, matPutMatrix, matPutString,
     *                 mxGetPr, mxCreateSparse, mxCreateFull,
     *                 mxGetIr, mxGetJc
      INTEGER*4        Ap, bigp, bp, cp, lp, up, fp, ifreep
      double precision ifree(freemax)
      integer          nfree, stat
      data             nfree /0/
C  =============================================
      data             mone,zero,one,big/-1.d0,0.d0,1.d0,1.d32/
      data             objdone /.false./
c
c     generic MPS format statement
    1 format(3a1,1x,a8,2x,a8,2x,e12.0,3x,a8,2x,e12.0)
c
      open(unit=3,status='OLD',file='in.mps')
c
c-----
c
c     read problem name
c
      call rdc1('NAME',4)
      backspace 3
      read(3,2) title
    2 format(a80)
      write(*,*) ' NAME    done'
c
c-----
c
c     read row kinds and row names
c
      call rdc1('ROWS',4)
      objnam = '        '
      m = 0
      n = 0
      na = 0
   10 continue
      read(3,1) test,c1,r1,v1,r2,v2
      if (test(1) .ne. ' ') then
        backspace 3
        go to 30
      endif
      m = m+1
      if (m .ge. mmax) go to 200
      rownam(m) = c1
      if (m .eq. (m/1000)*1000) write(*,*) '   m = ',m,' row = ',c1
      if (test(2) .eq. 'N' .or. test(3) .eq. 'N') then
        if (objnam .ne. '        ' .or. objdone) then
          write(*,*)'   row ',c1,' ignored : objective row is ',objnam
          junk = c1
          m = m - 1
c         krow(m) = 1
        else
          objnam = c1
          m = m - 1
          objdone = .true.
        endif
      else if (test(2) .eq. 'G' .or. test(3) .eq. 'G') then
        krow(m) = 2
        n = n+1
        na = na + 1
        ja(n) = na
        ia(na) = m
        A(na) = mone
        c(n) = zero
        l(n) = zero
        u(n) = big
        colnam(n) = c1
      else if (test(2) .eq. 'L' .or. test(3) .eq. 'L') then
        krow(m) = 3
        n = n+1
        na = na + 1
        ja(n) = na
        ia(na) = m
        A(na) = one
        c(n) = zero
        l(n) = zero
        u(n) = big
        colnam(n) = c1
      else if (test(2) .eq. 'E' .or. test(3) .eq. 'E') then
        krow(m) = 4
      else
        write(*,*) 'data for row ',m,' must start with N,G,L, or E'
        stop
      endif
      go to 10
   30 continue
      if (objnam .eq. '        ') then
          write(*,*)'No rows are kind N'
          write(*,*)'Exactly one such row is required'
        stop
      endif
      call sort(m,rownam,irow)
      write(*,*) ' ROWS    done'
c
c-----
c
c     read column names, associated row names, and matrix entries
c
      call rdc1('COLUMNS',7)
      c0 = '        '
      n0 = n
      lsort = .false.
   40 continue
      read(3,1) test,c1,r1,v1,r2,v2
        if ((c1.ne.c0) .and. lsort) 
     *       call srtcol(lsort,colnam(n),ja(n),na,ia,A)
      if (test(1) .ne. ' ') then
        backspace 3
        go to 50
      endif
      if (c1 .ne. c0) then
        n = n + 1
        if (n .gt. nmax) go to 200
        c0 = c1
        i0 = 0
        colnam(n) = c1
        if (n .eq. (n/1000)*1000) write(*,*) '   n = ',n,' column = ',c1
        ja(n) = na+1
        c(n) = zero
        l(n) = zero
        u(n) = big 
      endif
      if (r1 .eq. objnam) then
        c(n) = v1
      else
        if (r1 .ne. junk) call search(r1,rownam,irow,1,m,i)
        if (i.gt.0 .and. krow(i).ne.1) then
          if (i .le. i0) lsort = .true.
          na = na + 1
          if (na .gt. namax) go to 200
          ia(na) = i
          A(na) = v1
          i0 = i
        endif
      endif
      if (r2 .eq. objnam) then
        c(n) = v2
      else
        if (r2 .ne. junk) call search(r2,rownam,irow,1,m,i)
        if (i.gt.0 .and. krow(i).ne.1) then
          if (i .le. i0) lsort = .true.
          na = na + 1
          if (na .gt. namax) go to 200
          ia(na) = i
          A(na) = v2
          i0 = i
        endif
      endif
      go to 40
   50 continue
      call sort(n0,colnam,jcol)
      call sort(n-n0,colnam(n0+1),jcol(n0+1))
      write(*,*) ' COLUMNS done'
c
c-----
c
c     read right hand side row names, and entries 
c
      call rdc1('RHS',3)
      do 52 i=1,m
        b(i) = 0.d0
   52 continue
   60 continue
      read(3,1) test,c1,r1,v1,r2,v2
      if (test(1) .ne. ' ') then
        backspace 3
        go to 70
      endif
      call search(r1,rownam,irow,1,m,i)
      if (i .gt. 0) b(i) = v1
      call search(r2,rownam,irow,1,m,i)
      if (i .gt. 0) b(i) = v2
      go to 60
   70 continue
      write(*,*) ' RHS     done'
c
c-----
c
c     read ranges
c
      if (rdc2('RANGES',6)) then
   80 continue
      read(3,1) test,c1,r1,v1,r2,v2
      if (test(1) .ne. ' ') then
        backspace 3
        go to 82
      endif
      call search(r1,rownam,irow,1,m,i)
      if (i .gt. 0) then
        if (krow(i) .eq. 2 .or. krow(i) .eq. 3) then
          call search(r1,colnam,jcol,1,n0,j)
          u(j) = dabs(v1)
        else
          write(*,*) ' rdmps : no ranges allowed for krow =',krow(i)
          stop
        endif
      endif
      call search(r2,rownam,irow,1,m,i)
      if (i .gt. 0) then
        if (krow(i) .eq. 2 .or. krow(i) .eq. 3) then
          call search(r2,colnam,jcol,1,n0,j)
          u(j) = dabs(v2)
        else
          write(*,*) ' rdmps : no ranges allowed for krow =',krow(i)
          stop
        endif
      endif
      go to 80
   82 continue
      endif
      write(*,*) ' RANGES  done'
c
c-----
c
c     read bounds
c
      if (rdc2('BOUNDS',6)) then
        n1 = n
   84   continue
      read(3,1) test,c1,r1,v1,r2,v2
      if (test(1) .ne. ' ') then
        backspace 3
        go to 86
      endif
      if (test(2) .eq. 'L' .and. test(3) .eq. 'O') then
        call search(r1,colnam(n0+1),jcol(n0+1),1,n-n0,j)
        if (j .gt. 0) l(n0+j) = v1
      else if (test(2) .eq. 'U' .and. test(3) .eq. 'P') then
        call search(r1,colnam(n0+1),jcol(n0+1),1,n-n0,j)
        if (j .gt. 0) u(n0+j) = v1
      else if (test(2) .eq. 'F' .and. test(3) .eq. 'X') then
        call search(r1,colnam(n0+1),jcol(n0+1),1,n-n0,j)
        if (j .gt. 0) l(n0+j) = v1
        if (j .gt. 0) u(n0+j) = v1
      else if (test(2) .eq. 'F' .and. test(3) .eq. 'R') then
        call search(r1,colnam(n0+1),jcol(n0+1),1,n-n0,j)
        if (j .gt. 0) then 
          j = n0+j
          write(*,*) ' free variable : ',r1
C  =====================================
          if (nfree .gt. freemax) go to 200
          ifree(nfree+1) = j
          ifree(nfree+2) = n1 + 1
          nfree = nfree+2
C  =====================================
          n1 = n1 + 1
          if (n1 .gt. nmax) go to 200
          colnam(n1) = r1
          ja(n1) = na+1
          c(n1) = -c(j)
          l(n1) = zero
          u(n1) = big
          do 85 ij=ja(j),ja(j+1)-1
            na = na + 1
            if (na .gt. namax) go to 200
            ia(na) = ia(ij)
            A(na) = -A(ij)
   85     continue
        endif
      else
        write(*,*) ' rdmps: expected bounds type LO,UP,FX,FR :',
     *             ' found ',test(2),test(3), ' ignored'
C    *             ' found ',test(2),test(3)
C       stop
      endif
      go to 84
   86 continue
      n = n1
      endif
      write(*,*) ' BOUNDS  done'
c
c-----
c
c     read end-of-file card
c
      call rdc1('ENDATA',6)
c
c-----
c
c     special check to maximize 'PROFIT  '
c
      if (objnam .eq. 'PROFIT  ') then
        do 100 j=1,n
          c(j) = -c(j)
  100   continue
      endif
      write(*,*) ' reading done'
c
c-----
c
c     change indexing to C convention 0:n-1
c
        do 120 j=1,na
          ia(j) = ia(j)-1
  120   continue
        do 140 j=1,n
          ja(j) = ja(j)-1
  140   continue
        ja(n+1) = na
c
C  =====================================
c     write out
c
      fp = matOpen('default.mat', 'w')
      stat = matPutString(fp, "NAME", title)

      bigp = mxCreateFull(1, 1, 0)
      call mxSetName(bigp, 'BIG')
      call mxCopyReal8ToPtr(big, mxGetPr(bigp), 1)
      stat = matPutMatrix(fp, bigp)
      call mxFreeMatrix(bigp)

      Ap = mxCreateSparse(m, n, na, 0)
      call mxSetName(Ap, 'A')
      call mxCopyReal8ToPtr(A, mxGetPr(Ap), na)
      call mxCopyInteger4ToPtr(ia, mxGetIr(Ap), na)
      call mxCopyInteger4ToPtr(ja, mxGetJc(Ap), n+1)
      stat = matPutMatrix(fp, Ap)
      call mxFreeMatrix(Ap)

      bp = mxCreateFull(m, 1, 0)
      call mxSetName(bp, 'b')
      call mxCopyReal8ToPtr(b, mxGetPr(bp), m)
      stat = matPutMatrix(fp, bp)
      call mxFreeMatrix(bp)

      cp = mxCreateFull(n, 1, 0)
      call mxSetName(cp, 'c')
      call mxCopyReal8ToPtr(c, mxGetPr(cp), n)
      stat = matPutMatrix(fp, cp)
      call mxFreeMatrix(cp)

      lp = mxCreateFull(n, 1, 0)
      call mxSetName(lp, 'lbounds')
      call mxCopyReal8ToPtr(l, mxGetPr(lp), n)
      stat = matPutMatrix(fp, lp)
      call mxFreeMatrix(lp)

      up = mxCreateFull(n, 1, 0)
      call mxSetName(up, 'ubounds')
      call mxCopyReal8ToPtr(u, mxGetPr(up), n)
      stat = matPutMatrix(fp, up)
      call mxFreeMatrix(up)

      ifreep = mxCreateFull(nfree, 1, 0)
      call mxSetName(ifreep, 'ifree')
      call mxCopyReal8ToPtr(ifree, mxGetPr(ifreep), nfree)
      stat = matPutMatrix(fp, ifreep)
      call mxFreeMatrix(ifreep)

      stat = matClose(fp)
      write(*,*) ' mps2mat done'
      stop
C  =====================================
c
c-----
c
c     error messages
c
  200 continue
      m = mmax - 1
      if (m.gt.mmax) write(*,*)
     *               ' rdmps : too many rows. limit = ',m
      if (n.gt.nmax) write(*,*)
     *               ' rdmps : too many columns. limit = ',nmax
      if (na.gt.namax) write(*,*)
     *               ' rdmps : too many matrix entries. limit = ',namax
C  =====================================
      if (nfree.gt.freemax) write(*,*)
     *             ' rdmps : too many free variables. limit = ',freemax
C  =====================================
      stop
      end
c
c-----------------------------------------------------------------------
c
      subroutine  rdc1(word,nword)
      integer     nword,i
      character*1 word(8),cword(53)
   10 continue
      read(3,20) cword
   20 format(80a1)
      do 30 i=1,nword
        if (cword(i) .ne. word(i)) then
          write(*,*) ' expected ',word,' , found ',cword
          go to 10
c         stop
        endif
   30 continue
      return
      end
c
c-----------------------------------------------------------------------
c
      logical function rdc2(word,nword)
      integer          nword,i
      character*1      word(8),cword(8)
      read(3,10) cword
   10 format(8a1)
      do 20 i=1,nword
        if (cword(i) .ne. word(i)) then
          rdc2 = .false.
          backspace 3
          return
        endif
   20 continue
      rdc2 = .true.
      return
      end
c
c-----------------------------------------------------------------------
c
      subroutine       srtcol(lsort,colnam,i1,i2,ia,A)
      character*8      colnam
      integer          i1,i2,ia(i2),i,j,k
      double precision A(i2),t
      logical lsort
      lsort = .false.
      do 20 i=i1,i2
        k = i
        do 10 j=i+1,i2
          if (ia(j) .eq. ia(k)) then
            write(*,*) 'srtcol : duplicate entries for column ',colnam
            stop
          endif
          if (ia(j) .lt. ia(k))  k = j
   10   continue
        if (k .ne. i) then
          j = ia(k)
          ia(k) = ia(i)
          ia(i) = j
          t = A(k)
          A(k) = A(i)
          A(i) = t
        endif
   20 continue
      return
      end
c                                                         
c-----------------------------------------------------------------------
c
      subroutine  search(word,list,iptr,ilo0,ihi0,iword)
      integer     iword,ilo0,ihi0,ilo,ihi,iptr(ihi0)
      character*8 word,list(ihi0)
      iword = 0    
      if (word .eq. '        ') return
      ilo = ilo0 
      ihi = ihi0
   10 continue
      if (ilo .ge. ihi) then
        if (word .eq. list(iptr(ilo))) then
          iword = iptr(ilo)
          return
        else
          ihi = max0(1,ihi)
          write(*,*) ' search : word = ',word,' not found on list'
          write(*,*) '          nearest entry = ',list(iptr(ihi))
CZ ----- 1/4/96 ----
CZ        stop
          iword = 0
          return
CZ ----- 1/4/96 ----
        endif
      endif
      iword = (ilo+ihi)/2
      if (word .lt. list(iptr(iword))) then
        ihi = min0(iword,ihi-1)
        go to 10
      else if (word .gt. list(iptr(iword))) then
        ilo = max0(iword,ilo+1)
        go to 10
      else
        iword = iptr(iword)
        return
      endif
      end
c
c-----------------------------------------------------------------------
c
      subroutine  sort(n,list,iptr)
      integer     n,iptr(n),offset,limit,switch,i,isave
      character*8 list(n)
c
c     shell sort from Microsoft QuickBasic -
c     worst case O(n*n), average O(n*log(n))
c
      do 5 i=1,n
        iptr(i) = i
    5 continue
c
      offset = n/2
   10 continue
        if (offset .le. 0) go to 40
        limit = n - offset
   20   continue
          switch = 0
          do 30 i=1,limit
            if (list(iptr(i)) .gt. list(iptr(i+offset))) then
              isave = iptr(i)
              iptr(i) = iptr(i+offset)
              iptr(i+offset) = isave
              switch = i
            endif
   30     continue
          limit = switch - offset
        if (switch .ne. 0) go to 20
        offset = offset / 2
      go to 10
c
c     check for duplicates
c
   40 continue
      do 50 i=1,n-1
        if (list(iptr(i)) .eq. list(iptr(i+1))) then
          write(*,*) 'sort : duplicate entry ',list(iptr(i))
          stop
        endif
   50 continue
c
      return
      end
